var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
import { Subject } from 'rxjs';
import { extractHostnameAndPort, generateId, generateKey, isBrowser, isOnline, isURL, isVisible, log, validateKey, } from './misc/util';
import { ChannelBuilder } from './service/channelBuilder/ChannelBuilder';
import { FullMesh } from './service/topology/FullMesh';
import { TopologyEnum, TopologyState } from './service/topology/Topology';
import { UserMessage } from './service/UserMessage';
import { Signaling, SignalingState } from './Signaling';
import { WebChannelState } from './WebChannelState';
import { WebSocketBuilder } from './WebSocketBuilder';
export var webChannelDefaultOptions = {
    topology: TopologyEnum.FULL_MESH,
    signalingServer: 'wss://signaling.netflux.coedit.re',
    rtcConfiguration: {
        iceServers: [{ urls: 'stun:stun3.l.google.com:19302' }],
    },
    autoRejoin: true,
};
var REJOIN_TIMEOUT = 3000;
var WebChannel = (function () {
    function WebChannel(options) {
        this.STREAM_ID = 2;
        var fullOptions = Object.assign({}, webChannelDefaultOptions, options);
        this.streamSubject = new Subject();
        this.topologyEnum = fullOptions.topology;
        this.autoRejoin = fullOptions.autoRejoin;
        this.rtcConfiguration = fullOptions.rtcConfiguration;
        this.members = [];
        this.id = 0;
        this.key = '';
        this.myId = 0;
        this.state = WebChannelState.LEFT;
        this.rejoinEnabled = false;
        this.rejoinTimer = undefined;
        this.topology = {};
        this._onAlone = function () { };
        this.onMemberJoin = function none() { };
        this.onMemberLeave = function none() { };
        this.onMessage = function none() { };
        this.onStateChange = function none() { };
        this.onSignalingStateChange = function none() { };
        this.userMsg = new UserMessage();
        this.signaling = new Signaling(this, fullOptions.signalingServer);
        this.subscribeToSignalingState();
        this.webSocketBuilder = new WebSocketBuilder(this);
        this.channelBuilder = new ChannelBuilder(this);
        this.setTopology(fullOptions.topology);
        if (isBrowser) {
            this.subscribeToBrowserEvents();
        }
    }
    Object.defineProperty(WebChannel.prototype, "messageFromStream", {
        get: function () {
            return this.streamSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebChannel.prototype, "onAlone", {
        set: function (handler) {
            this._onAlone = handler;
        },
        enumerable: true,
        configurable: true
    });
    WebChannel.prototype.sendOverStream = function (msg) {
        this.topology.sendTo(msg);
    };
    WebChannel.prototype.join = function (key) {
        if (key === void 0) { key = generateKey(); }
        validateKey(key);
        if (this.state === WebChannelState.LEFT) {
            this.startJoin(key);
        }
    };
    WebChannel.prototype.invite = function (url) {
        if (isURL(url)) {
            var hostnamePort = extractHostnameAndPort(url);
            try {
                for (var _a = __values(this.topology.neighbors), _b = _a.next(); !_b.done; _b = _a.next()) {
                    var ch = _b.value;
                    if (hostnamePort === extractHostnameAndPort(ch.url)) {
                        return;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                }
                finally { if (e_1) throw e_1.error; }
            }
            this.webSocketBuilder
                .connectToInvite(url)
                .catch(function (err) { return log.webgroup("Failed to invite the bot " + url + ": " + err.message); });
        }
        else {
            throw new Error("Failed to invite a bot: " + url + " is not a valid URL");
        }
        var e_1, _c;
    };
    WebChannel.prototype.leave = function () {
        if (this.state !== WebChannelState.LEFT) {
            this.rejoinEnabled = false;
            this.key = '';
            this.signaling.close();
            this.topology.leave();
            this.clean();
            this.setState(WebChannelState.LEFT);
        }
    };
    WebChannel.prototype.send = function (data) {
        if (this.members.length !== 1) {
            try {
                for (var _a = __values(this.userMsg.encodeUserMessage(data)), _b = _a.next(); !_b.done; _b = _a.next()) {
                    var chunk = _b.value;
                    this.topology.send({
                        senderId: this.myId,
                        recipientId: 0,
                        serviceId: UserMessage.SERVICE_ID,
                        content: chunk,
                    });
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                }
                finally { if (e_2) throw e_2.error; }
            }
        }
        var e_2, _c;
    };
    WebChannel.prototype.sendTo = function (id, data) {
        if (this.members.length !== 1) {
            try {
                for (var _a = __values(this.userMsg.encodeUserMessage(data)), _b = _a.next(); !_b.done; _b = _a.next()) {
                    var chunk = _b.value;
                    this.topology.sendTo({
                        senderId: this.myId,
                        recipientId: id,
                        serviceId: UserMessage.SERVICE_ID,
                        content: chunk,
                    });
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                }
                finally { if (e_3) throw e_3.error; }
            }
        }
        var e_3, _c;
    };
    WebChannel.prototype.onMemberJoinProxy = function (id) {
        if (!this.members.includes(id)) {
            this.members[this.members.length] = id;
            this.onMemberJoin(id);
        }
    };
    WebChannel.prototype.onAdjacentMembersLeaveProxy = function (ids) {
        var _this = this;
        if (ids.length !== 0) {
            ids.forEach(function (id) {
                if (_this.members.includes(id)) {
                    _this.members.splice(_this.members.indexOf(id), 1);
                    _this.onMemberLeave(id);
                }
            });
            if (this.members.length === 1) {
                this._onAlone();
                this.topology.setLeftState();
            }
            else if (this.signaling.state === SignalingState.CHECKED &&
                this.topology.state === TopologyState.JOINED) {
                this.signaling.check();
            }
        }
    };
    WebChannel.prototype.onDistantMembersLeaveProxy = function (ids) {
        var _this = this;
        ids.forEach(function (id) {
            if (_this.members.includes(id)) {
                _this.members.splice(_this.members.indexOf(id), 1);
                _this.onMemberLeave(id);
            }
        });
    };
    WebChannel.prototype.init = function (key, id) {
        if (id === void 0) { id = generateId(); }
        this.id = id;
        this.myId = generateId();
        this.members = [this.myId];
        this.key = key;
        this.rejoinEnabled = this.autoRejoin;
        if (this.rejoinTimer) {
            global.clearTimeout(this.rejoinTimer);
            this.rejoinTimer = undefined;
        }
        this.setState(WebChannelState.JOINING);
    };
    WebChannel.prototype.clean = function () {
        if (this.rejoinTimer) {
            global.clearTimeout(this.rejoinTimer);
            this.rejoinTimer = undefined;
        }
        this.channelBuilder.clean();
        this.userMsg.clean();
        this.members = [];
        this.id = 0;
        this.myId = 0;
    };
    WebChannel.prototype.setState = function (state) {
        if (this.state !== state) {
            log.webGroupState(WebChannelState[state], this.myId);
            this.state = state;
            this.onStateChange(state);
        }
    };
    WebChannel.prototype.setTopology = function (topologyEnum) {
        var _this = this;
        this.topologyEnum = topologyEnum;
        this.topology = new FullMesh(this);
        this.topology.onState.subscribe(function (state) {
            switch (state) {
                case TopologyState.JOINING:
                    _this.setState(WebChannelState.JOINING);
                    if (_this.signaling.state === SignalingState.CLOSED) {
                        _this.signaling.connect(_this.key);
                    }
                    break;
                case TopologyState.JOINED:
                    _this.setState(WebChannelState.JOINED);
                    if (_this.signaling.state === SignalingState.OPEN ||
                        _this.signaling.state === SignalingState.CHECKED) {
                        _this.signaling.check();
                    }
                    break;
                case TopologyState.LEFT:
                    switch (_this.signaling.state) {
                        case SignalingState.CLOSED:
                            _this.rejoinOrLeave();
                            break;
                        case SignalingState.CONNECTING:
                            _this.setState(WebChannelState.JOINING);
                            break;
                        case SignalingState.OPEN:
                            _this.setState(WebChannelState.JOINING);
                            _this.signaling.check();
                            break;
                        case SignalingState.CHECKING:
                            _this.setState(WebChannelState.JOINING);
                            break;
                        case SignalingState.CHECKED:
                            _this.signaling.check();
                            break;
                    }
                    break;
            }
        });
    };
    WebChannel.prototype.subscribeToSignalingState = function () {
        var _this = this;
        this.signaling.onState.subscribe(function (state) {
            log.signalingState(SignalingState[state], _this.myId);
            _this.onSignalingStateChange(state);
            switch (state) {
                case SignalingState.CLOSED:
                    if (_this.topology.state === TopologyState.LEFT) {
                        _this.rejoinOrLeave();
                    }
                    else if (_this.topology.state === TopologyState.JOINED && _this.rejoinEnabled) {
                        _this.signaling.connect(_this.key);
                    }
                    break;
                case SignalingState.OPEN:
                    if (_this.topology.state !== TopologyState.JOINING) {
                        _this.signaling.check();
                    }
                    break;
                case SignalingState.CHECKED:
                    if (_this.state === WebChannelState.JOINING &&
                        _this.topology.state === TopologyState.LEFT &&
                        _this.signaling.connected) {
                        _this.topology.setJoinedState();
                    }
                    break;
            }
        });
    };
    WebChannel.prototype.subscribeToBrowserEvents = function () {
        var _this = this;
        global.addEventListener('online', function () {
            if (_this.state === WebChannelState.LEFT &&
                isVisible() &&
                !_this.rejoinTimer &&
                _this.rejoinEnabled) {
                _this.startJoin();
            }
        });
        global.addEventListener('visibilitychange', function () {
            if (isVisible() &&
                _this.state === WebChannelState.LEFT &&
                isOnline() &&
                !_this.rejoinTimer &&
                _this.rejoinEnabled) {
                _this.startJoin();
            }
        });
        global.addEventListener('beforeunload', function () { return _this.leave(); });
    };
    WebChannel.prototype.startJoin = function (key) {
        if (key === void 0) { key = this.key; }
        this.init(key);
        this.signaling.connect(key);
    };
    WebChannel.prototype.rejoinOrLeave = function () {
        var _this = this;
        this.clean();
        if (this.rejoinEnabled) {
            this.init(this.key);
            this.rejoinTimer = global.setTimeout(function () {
                if (_this.state === WebChannelState.LEFT &&
                    isVisible() &&
                    isOnline() &&
                    _this.rejoinEnabled) {
                    _this.startJoin();
                }
            }, REJOIN_TIMEOUT);
        }
        else {
            this.setState(WebChannelState.LEFT);
        }
    };
    return WebChannel;
}());
export { WebChannel };
//# sourceMappingURL=WebChannel.js.map