var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import { ChannelType, createHeartbeatMsg, MAXIMUM_MISSED_HEARTBEAT, } from '../../Channel';
import { log } from '../../misc/util';
import { fullMesh as proto } from '../../proto';
import { Topology, TopologyState } from './Topology';
var REQUEST_MEMBERS_INTERVAL = 6000;
var MAX_ROUTE_DISTANCE = 3;
var HEARTBEAT_INTERVAL = 3000;
var FullMesh = (function (_super) {
    __extends(FullMesh, _super);
    function FullMesh(wc) {
        var _this = _super.call(this, wc, FullMesh.SERVICE_ID, proto.Message) || this;
        _this.adjacentMembers = new Map();
        _this.distantMembers = new Map();
        _this.membersRequestEncoded = _super.prototype.encode.call(_this, { membersRequest: true });
        _this.wcStream.message.subscribe(function (_a) {
            var channel = _a.channel, senderId = _a.senderId, msg = _a.msg;
            return _this.handleServiceMessage(channel, senderId, msg);
        });
        _this.wc.channelBuilder.onChannel.subscribe(function (ch) {
            log.topology('Adding new adjacent member: ', ch.id);
            _this.distantMembers.delete(ch.id);
            var member = _this.adjacentMembers.get(ch.id);
            if (member) {
                log.topology('Replacing the same channel');
                member.close();
            }
            _this.adjacentMembers.set(ch.id, ch);
            _this.wc.onMemberJoinProxy(ch.id);
            if (_this.adjacentMembers.size === 1 && _this.membersRequestInterval === undefined) {
                _this.startIntervals();
            }
            if (ch.type === ChannelType.JOINING) {
                _super.prototype.setState.call(_this, TopologyState.JOINING);
                var members = ch.initData.members;
                _this.connectToMany(members, ch.id).then(function () {
                    _this.membersRequest();
                    _super.prototype.setState.call(_this, TopologyState.JOINED);
                });
            }
            else {
                _this.notifyDistantMembers();
            }
        });
        global.fullmesh = function () {
            log.topology('Fullmesh info:', {
                myId: _this.wc.myId,
                signalingState: _this.wc.signaling.state,
                webGroupState: _this.wc.state,
                topologyState: TopologyState[_this.state],
                adjacentMembers: Array.from(_this.adjacentMembers.keys()).toString(),
                distantMembers: Array.from(_this.distantMembers.keys()).toString(),
                distantMembersDETAILS: Array.from(_this.distantMembers.keys()).map(function (id) {
                    var adjacentMembers = _this.distantMembers.get(id);
                    return { id: id, adjacentMembers: adjacentMembers ? adjacentMembers.adjacentIds : [] };
                }),
            });
        };
        return _this;
    }
    FullMesh.prototype.send = function (msg) {
        var _this = this;
        this.adjacentMembers.forEach(function (ch) { return ch.encodeAndSend(msg); });
        this.distantMembers.forEach(function (distantMember, id) {
            _this.sendToDistantPeer(distantMember, Object.assign(msg, { recipientId: id }));
        });
    };
    FullMesh.prototype.sendTo = function (msg) {
        var ch = this.adjacentMembers.get(msg.recipientId);
        if (ch) {
            ch.encodeAndSend(msg);
        }
        else {
            this.sendToDistantPeer(this.distantMembers.get(msg.recipientId), msg);
        }
    };
    FullMesh.prototype.forward = function (msg) {
        this.sendTo(msg);
    };
    FullMesh.prototype.leave = function () {
        if (this.state !== TopologyState.LEFT) {
            this.clean();
            this.adjacentMembers.forEach(function (ch) { return ch.close(); });
            this.wc.onAdjacentMembersLeaveProxy(Array.from(this.adjacentMembers.keys()));
            _super.prototype.setState.call(this, TopologyState.LEFT);
        }
    };
    FullMesh.prototype.onChannelClose = function (channel) {
        this.adjacentMembers.delete(channel.id);
        if (this.adjacentMembers.size === 0) {
            this.clean();
        }
        else {
            this.notifyDistantMembers();
        }
        this.wc.onAdjacentMembersLeaveProxy([channel.id]);
    };
    Object.defineProperty(FullMesh.prototype, "neighbors", {
        get: function () {
            return Array.from(this.adjacentMembers.values());
        },
        enumerable: true,
        configurable: true
    });
    FullMesh.prototype.clean = function () {
        this.wc.onDistantMembersLeaveProxy(Array.from(this.distantMembers.keys()));
        this.distantMembers.clear();
        global.clearInterval(this.heartbeatInterval);
        this.heartbeatInterval = undefined;
        global.clearInterval(this.membersRequestInterval);
        this.membersRequestInterval = undefined;
    };
    FullMesh.prototype.handleServiceMessage = function (channel, senderId, msg) {
        switch (msg.type) {
            case 'membersResponse': {
                this.connectToMany(msg.membersResponse.ids, senderId);
                break;
            }
            case 'membersRequest': {
                channel.encodeAndSend({
                    recipientId: channel.id,
                    serviceId: FullMesh.SERVICE_ID,
                    content: _super.prototype.encode.call(this, { membersResponse: { ids: this.wc.members } }),
                });
                break;
            }
            case 'adjacentMembers': {
                var ids = msg.adjacentMembers.ids;
                if (!this.adjacentMembers.has(senderId)) {
                    var distantMember = this.distantMembers.get(senderId);
                    if (distantMember) {
                        distantMember.adjacentIds = ids;
                    }
                    else {
                        this.distantMembers.set(senderId, {
                            adjacentIds: ids,
                            missedHeartbeat: 0,
                        });
                        this.wcStream.send({ adjacentMembers: { ids: Array.from(this.adjacentMembers.keys()) } }, senderId);
                    }
                }
                break;
            }
            case 'heartbeat': {
                var distantMember = this.distantMembers.get(senderId);
                if (distantMember) {
                    distantMember.missedHeartbeat = 0;
                }
                break;
            }
        }
    };
    FullMesh.prototype.connectToMany = function (ids, adjacentId) {
        var _this = this;
        var missingIds = ids.filter(function (id) { return !_this.adjacentMembers.has(id) && id !== _this.wc.myId; });
        if (missingIds.length !== 0) {
            var msg_1 = _super.prototype.encode.call(this, {
                adjacentMembers: { ids: Array.from(this.adjacentMembers.keys()) },
            });
            var attempts_1 = [];
            missingIds.forEach(function (id) {
                if (!_this.distantMembers.has(id)) {
                    _this.distantMembers.set(id, { adjacentIds: [adjacentId], missedHeartbeat: 0 });
                }
                _this.wcStream.send(msg_1, id);
                attempts_1[attempts_1.length] = _this.wc.channelBuilder
                    .connectOverWebChannel(id)
                    .catch(function (err) {
                    if (err.message !== 'ping') {
                        _this.wc.onMemberJoinProxy(id);
                    }
                });
            });
            return Promise.all(attempts_1);
        }
        return Promise.resolve();
    };
    FullMesh.prototype.membersRequest = function () {
        if (this.adjacentMembers.size !== 0) {
            var index = Math.floor(Math.random() * this.adjacentMembers.size);
            var iterator = this.adjacentMembers.values();
            for (var i = 0; i < index; i++) {
                iterator.next();
            }
            var channel = iterator.next().value;
            channel.encodeAndSend({
                recipientId: channel.id,
                serviceId: FullMesh.SERVICE_ID,
                content: this.membersRequestEncoded,
            });
        }
    };
    FullMesh.prototype.notifyDistantMembers = function () {
        var _this = this;
        if (this.distantMembers.size !== 0) {
            var msg_2 = _super.prototype.encode.call(this, {
                adjacentMembers: { ids: Array.from(this.adjacentMembers.keys()) },
            });
            this.distantMembers.forEach(function (dm, id) { return _this.wcStream.send(msg_2, id); });
        }
    };
    FullMesh.prototype.startIntervals = function () {
        var _this = this;
        this.membersRequestInterval = global.setInterval(function () { return _this.membersRequest(); }, REQUEST_MEMBERS_INTERVAL);
        this.heartbeatInterval = global.setInterval(function () {
            _this.adjacentMembers.forEach(function (ch) { return ch.sendHeartbeat(); });
            _this.distantMembers.forEach(function (peer, id) {
                peer.missedHeartbeat++;
                if (peer.missedHeartbeat >= MAXIMUM_MISSED_HEARTBEAT + 1) {
                    log.topology("Distant peer " + id + " has left: too many missed heartbeats");
                    _this.distantMembers.delete(id);
                    _this.wc.onDistantMembersLeaveProxy([id]);
                }
                _this.wcStream.send(createHeartbeatMsg(_this.wc.id, id));
            });
        }, HEARTBEAT_INTERVAL);
    };
    FullMesh.prototype.sendToDistantPeer = function (distantMember, msg) {
        for (var d = 1; d <= MAX_ROUTE_DISTANCE; d++) {
            var ch = this.findRoutedChannel(distantMember, d);
            if (ch) {
                ch.encodeAndSend(msg);
                return;
            }
        }
    };
    FullMesh.prototype.findRoutedChannel = function (distantMember, distance) {
        if (distantMember) {
            try {
                for (var _a = __values(this.adjacentMembers), _b = _a.next(); !_b.done; _b = _a.next()) {
                    var _c = __read(_b.value, 2), neighbourId = _c[0], ch = _c[1];
                    if (distantMember.adjacentIds.includes(neighbourId)) {
                        return ch;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_b && !_b.done && (_d = _a.return)) _d.call(_a);
                }
                finally { if (e_1) throw e_1.error; }
            }
            if (distance !== 0) {
                try {
                    for (var _e = __values(distantMember.adjacentIds), _f = _e.next(); !_f.done; _f = _e.next()) {
                        var id = _f.value;
                        var ch = this.findRoutedChannel(this.distantMembers.get(id), distance - 1);
                        if (ch) {
                            return ch;
                        }
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_f && !_f.done && (_g = _e.return)) _g.call(_e);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            }
        }
        return undefined;
        var e_1, _d, e_2, _g;
    };
    FullMesh.SERVICE_ID = 74315;
    return FullMesh;
}(Topology));
export { FullMesh };
//# sourceMappingURL=FullMesh.js.map